"""UK specific code."""
import logging
from datetime import datetime

import requests

from ..classes.programme import Programme
from ..const import RESPONSE_OK, SKY_STATUS_LIVE
from .const_gb import (CHANNEL_IMAGE_URL, LIVE_IMAGE_URL, PVR_IMAGE_URL,
                       SCHEDULE_URL, TERRITORY)

_LOGGER = logging.getLogger(__name__)


class SkyQCountry:
    """UK specific SkyQ."""

    def __init__(self):
        """Initialise UK remote."""
        self.pvr_image_url = PVR_IMAGE_URL

    def get_epg_data(self, sid, channelno, channel_name, epg_date):
        """Get EPG data for UK."""
        return self._get_data(sid, channelno, channel_name, epg_date)

    def build_channel_image_url(self, sid, channelname):
        """Build the channel image URL."""
        chid = "".join(e for e in channelname.casefold() if e.isalnum())
        return CHANNEL_IMAGE_URL.format(sid, chid)

    def _get_data(
        self, sid, channelno, channel_name, epg_date
    ):  # pylint: disable=unused-argument
        programmes = set()
        epg_data = self._get_epg_data(sid, epg_date)
        if epg_data is None:
            return programmes

        if len(epg_data) == 0:
            return programmes

        for programme in epg_data[0]["events"]:
            starttime = datetime.utcfromtimestamp(programme["st"])
            endtime = datetime.utcfromtimestamp(programme["st"] + programme["d"])
            title = programme["t"]
            season = None
            if "seasonnumber" in programme and programme["seasonnumber"] > 0:
                season = programme["seasonnumber"]
            episode = None
            if "episodenumber" in programme and programme["episodenumber"] > 0:
                episode = programme["episodenumber"]
            programmeuuid = None
            image_url = None
            if "programmeuuid" in programme:
                programmeuuid = str(programme["programmeuuid"])
                image_url = LIVE_IMAGE_URL.format(programmeuuid)

            eid = programme["eid"]
            programme = Programme(
                programmeuuid,
                starttime,
                endtime,
                title,
                season,
                episode,
                image_url,
                channel_name,
                SKY_STATUS_LIVE,
                "n/a",
                eid,
            )
            programmes.add(programme)

        return programmes

    def _get_epg_data(self, sid, epg_date):
        epg_date_str = epg_date.strftime("%Y%m%d")

        epg_url = SCHEDULE_URL.format(sid, epg_date_str)
        headers = {
            "x-skyott-territory": TERRITORY,
            "x-skyott-provider": "SKY",
            "x-skyott-proposition": "SKYQ",
        }
        resp = requests.get(epg_url, headers=headers)
        return resp.json()["schedule"] if resp.status_code == RESPONSE_OK else None



"""Italy specific code."""
import logging
from datetime import datetime, timedelta

import requests

from ..classes.programme import Programme
from ..const import RESPONSE_OK, SKY_STATUS_LIVE
from .const_it import (CHANNEL_IMAGE_URL, CHANNEL_URL, LIVE_IMAGE_URL,
                       PVR_IMAGE_URL, SCHEDULE_URL)

_LOGGER = logging.getLogger(__name__)


class SkyQCountry:
    """Italy specific SkyQ."""

    def __init__(self):
        """Initialise Italy remote."""
        self.pvr_image_url = PVR_IMAGE_URL
        self._channellist = None

        self._get_channels()

    def get_epg_data(self, sid, channelno, channel_name, epg_date):
        """Get EPG data for Italy."""
        epg_prev = epg_date - timedelta(days=1)
        query_date_from = epg_prev.strftime("%Y-%m-%dT22:00:00Z")
        query_date_to = epg_date.strftime("%Y-%m-%dT23:59:59Z")
        epg_data = self._get_data(
            sid, channelno, channel_name, query_date_from, query_date_to
        )

        midnight = datetime.combine(epg_date.date(), datetime.min.time())

        return [p for p in epg_data if p.endtime >= midnight]

    def build_channel_image_url(self, sid, channelname):
        """Build the channel image URL."""
        chid = "".join(e for e in channelname.casefold() if e.isalnum())
        return CHANNEL_IMAGE_URL.format(sid, chid)

    def _get_data(
        self, sid, channelno, channel_name, query_date_from, query_date_to
    ):  # pylint: disable=unused-argument
        programmes = set()

        try:
            epg_data = self._get_epg_data(channelno, query_date_from, query_date_to)
        except requests.exceptions.ConnectionError:
            return programmes

        if epg_data is None:
            return programmes

        if len(epg_data) == 0:
            return programmes

        epg_data_len = len(epg_data) - 1
        for index, programme in enumerate(epg_data):
            starttime = datetime.strptime(programme["starttime"], "%Y-%m-%dT%H:%M:%SZ")
            if index < epg_data_len:
                endtime_str = epg_data[index + 1]["starttime"]
            else:
                endtime_str = programme["endtime"]
            endtime = datetime.strptime(endtime_str, "%Y-%m-%dT%H:%M:%SZ")
            title = programme["eventTitle"]
            season = None
            if (
                "seasonNumber" in programme["content"]
                and programme["content"]["seasonNumber"] > 0
            ):
                season = programme["content"]["seasonNumber"]
            episode = None
            if (
                "episodeNumber" in programme["content"]
                and programme["content"]["episodeNumber"] > 0
            ):
                episode = programme["content"]["episodeNumber"]
            programmeuuid = None
            image_url = None
            if "uuid" in programme["content"]:
                programmeuuid = str(programme["content"]["uuid"])
                image_url = LIVE_IMAGE_URL.format(programmeuuid)

            programme = Programme(
                programmeuuid,
                starttime,
                endtime,
                title,
                season,
                episode,
                image_url,
                channel_name,
                SKY_STATUS_LIVE,
            )
            programmes.add(programme)

        return programmes

    def _get_channels(self):
        resp = requests.get(CHANNEL_URL)
        if resp.status_code == RESPONSE_OK:
            self._channellist = resp.json()["channels"]

    def _get_epg_data(self, channelno, query_date_from, query_date_to):

        cid = None
        for channel in self._channellist:
            if str(channel["number"]) == str(channelno):
                cid = channel["id"]

        epg_url = SCHEDULE_URL.format(cid, query_date_from, query_date_to)

        resp = requests.get(epg_url)

        return resp.json()["events"] if resp.status_code == RESPONSE_OK else None



"""DE specific code."""
import json
import logging
from datetime import datetime, timedelta, timezone

import pytz
import requests

from ..classes.programme import Programme
from ..const import RESPONSE_OK, SKY_STATUS_LIVE
from .const_de import (CHANNEL_IMAGE_URL, CHANNEL_URL, LIVE_IMAGE_URL,
                       PVR_IMAGE_URL, SCHEDULE_URL, TIMEZONE)

_LOGGER = logging.getLogger(__name__)


class SkyQCountry:
    """DE specific SkyQ."""

    def __init__(self):
        """Initialise DE remote."""
        self.pvr_image_url = PVR_IMAGE_URL
        self._channellist = None

        self._get_channels()

    def get_epg_data(self, sid, channelno, channel_name, epg_date):
        """Get EPG data for DE."""
        return self._get_data(sid, channelno, channel_name, epg_date)

    def build_channel_image_url(
        self, sid, channelname
    ):  # pylint: disable=unused-argument
        """Build the channel image URL."""
        for channel in self._channellist:
            if str(channel["sid"]) == str(sid):
                return CHANNEL_IMAGE_URL.format(channel["clu"])

    def _get_data(
        self, sid, channelno, channel_name, epg_date
    ):  # pylint: disable=unused-argument

        berlin_dt = epg_date.replace(tzinfo=pytz.utc).astimezone(
            pytz.timezone(TIMEZONE)
        )
        berlin_date = berlin_dt.strftime("%Y-%m-%dT")

        programmes = set()
        epg_data = self._get_epg_data(sid, epg_date)
        if epg_data is None:
            return programmes

        if len(epg_data) == 0:
            return programmes

        for programme in epg_data:
            starttimede = datetime.strptime(
                berlin_date + programme["bst"], "%Y-%m-%dT%H:%M"
            )
            starttime = (
                starttimede.replace(tzinfo=berlin_dt.tzinfo)
                .astimezone(pytz.utc)
                .replace(tzinfo=None)
            )
            endtime = starttime + timedelta(minutes=programme["len"])
            title = programme["et"]
            programmeuuid = str(programme["ei"])
            image_url = None
            if "pu" in programme:
                image_url = LIVE_IMAGE_URL.format(programme["pu"])
            elif "clu" in programme:
                image_url = LIVE_IMAGE_URL.format(programme["clu"])

            programme = Programme(
                programmeuuid,
                starttime,
                endtime,
                title,
                None,
                None,
                image_url,
                channel_name,
                SKY_STATUS_LIVE,
            )
            programmes.add(programme)

        return programmes

    def _get_channels(self):
        resp = requests.get(CHANNEL_URL)
        if resp.status_code == RESPONSE_OK:
            self._channellist = resp.json()

    def _get_epg_data(self, sid, epg_date):
        cid = None
        milli_date = int(epg_date.replace(tzinfo=timezone.utc).timestamp() * 1000)
        for channel in self._channellist:
            if str(channel["sid"]) == str(sid):
                cid = channel["ci"]

        epg_url = SCHEDULE_URL

        headers = {
            "Content-Type": 'application/json; charset="utf-8"',
        }
        payload = json.dumps(
            {
                "d": milli_date,
                "lt": 6,
                "t": 0,
                "s": 0,
                "pn": 1,
                "sto": 10,
                "epp": 50,
                "cil": [cid],
            }
        )

        resp = requests.post(
            epg_url,
            headers=headers,
            data=payload,
            verify=True,
            timeout=10,
        )
        return resp.json()["el"] if resp.status_code == RESPONSE_OK else None
